(function ($) {

  Drupal.simpleCrop = Drupal.simpleCrop || {};
  Drupal.simpleCrop.setCoords = Drupal.simpleCrop.setCoords || {};

  Drupal.behaviors.simpleCrop = {
    attach: function(context, settings) {

      settings.simpleCrop = settings.simpleCrop || {};
      settings.simpleCrop.images = settings.simpleCrop.images || {};

      jQuery.each(settings.simpleCrop.images, function(i, value) {

        // Get wrapper of image field.
        var $widget = $('.simplecrop-widget.image-' + value.fid, context);

        // Get unprocessed original image.
        var $original_img = $('.original-image img', $widget).filter(':not(.crop-processed)');

        // If this original unprocessed image exists - then process it.
        if ($original_img.length == 1) {

          // Initialize jcrop plugin.
          var jcrop = $.Jcrop($('.original-image img', $widget));

          // Get crop settings.
          var crop = value.settings;

          // Get scaled image width and height.
          crop.resolution = crop.resolution || [$original_img.attr('width'), $original_img.attr('height')];

          // Calculate image min/max crop area selections.
          crop.min_area = crop.min_area || [0, 0];
          crop.max_area = crop.max_area || crop.resolution;

          // Set default handlers to keep coords up to date.
          jcrop.setOptions({
            onChange: Drupal.simpleCrop.setCoords,
            onSelect: Drupal.simpleCrop.setCoords,
            minSize: crop.min_area,
            maxSize: crop.max_area
          });

          // Set aspect ratio (if exists).
          crop.ratio = crop.ratio || 0;
          if (crop.ratio) {
            jcrop.setOptions({
              aspectRatio: crop.ratio
            });
          }

          // Define initial crop area (if exists).
          crop.initial_area = crop.initial_area || false;
          if (crop.initial_area) {
            var coords = crop.initial_area;
            jcrop.setSelect([coords.x, coords.y, coords.x2, coords.y2]);
          }
          else {
            jcrop.release();
          }

          // Mark image as processed to avoid duplicated processments.
          $original_img.addClass('crop-processed');
        }

      });

      // Avoid issue with image caching by browsers.
      settings.simpleCrop.reload = settings.simpleCrop.reload || 0;
      if (settings.simpleCrop.reload) {
        var fid = settings.simpleCrop.reload;
        var $image = $('.simplecrop-widget.image-' + fid + ' .cropped-image img', context);
        if ($image.length) {

          // Add new argument to image url, so browser will send a new request for image.
          $image.attr('src', $image.attr('src') + '&random=' + Math.random());

          // Unset value of image that we need to reload.
          Drupal.settings.simpleCrop.reload = 0;
        }
      }

    }
  };

  Drupal.simpleCrop.setCoords = function(coords) {

    // Find a field wrapper for simplecrop widget.
    var $widget = $(this.ui.holder).parents('.simplecrop-widget');

    // Set an actual values in hidden inputs.
    $('.x', $widget).val(coords.x);
    $('.y', $widget).val(coords.y);
    $('.x2', $widget).val(coords.x2);
    $('.y2', $widget).val(coords.y2);
  }

})(jQuery);
